
from youtube_dl import YoutubeDL
from requests import get
from bs4 import BeautifulSoup
import re
import json

def spotifyparser(url):
    print("Pinging "+url)
    spotifyhtml=get(url)
    soup=BeautifulSoup(spotifyhtml.content,"lxml")
    tags=soup('script')
    x=re.findall("Spotify.Entity = (.*);",tags[5].contents[0])
    data=x[0]
    jsonfile=json.loads(data)

    print("Adding "+jsonfile['name']+" To Queue." )
    tracks=jsonfile['tracks']['items']

    tracklist=[]

    for track in tracks:
        trackname=track['track']['name']
        artistname=""
        for artist in track['track']['artists']:
            artistname=artistname+" "+artist['name']
        tracklist.append(trackname+artistname)

    return tracklist

class MyLogger(object):
    def debug(self, msg):
        if msg.startswith("[download] Downloading video"):
            numlist=re.findall('[0-9]+', msg)
            print("Processing Song: "+numlist[0]+"/"+numlist[1])
    def warning(self, msg):
        pass

    def error(self, msg):
        pass

def ytplaylistparser(url,beg):
    
    options={
    'logger': MyLogger(),
    'playlist_items':f'{beg}-{beg+4}'
    }
    print("Pinging Youtube")
    with YoutubeDL(options) as ytdl:
        try:
            meta = ytdl.extract_info(url, download=False)
        except:
            pass
    
    tracklist=[]

    tracker=0
    for song in meta['entries']:
        tracklist.append(song['webpage_url'])
        tracker=tracker+1
    if tracker==5:
        beg=beg+tracker
    else:
        beg=-1
    return tracklist,beg

def download(link):
    options={
    'format': 'aac/mp3/ogg/wav/3gp/m4a/mp4',
    'outtmpl': 'downloads/%(id)s',
    'quiet':'True'
    }
    print("Downloading")
    with YoutubeDL(options) as ytdl:
        try:
            get(link)
        except:
            meta=ytdl.extract_info(f"ytsearch:{link}", download=True)['entries'][0]
        else:
            meta=ytdl.extract_info(link, download=True)
    print("Done Downloading")
    return meta



import jakym.downloader as downloader
import jakym.player as player
import os , glob
import threading
from pyfiglet import Figlet
from termcolor import colored

class Playlist:

    def __init__(self):
        self.playlist=[]

    def playlistmanager(self):
        song=self.playlist.pop(0)
        return song

    def songqueuer(self,query):
        self.playlist.append(query+" song")

    def songdownloader(self,song):
        meta=downloader.download(song)
        return meta

    def songplayer(self,meta):
        print(colored("Currently Playing : " + meta['title'],'yellow'))
        player.playmusic('downloads/'+meta['id'])

musicplaylist = Playlist()

def cleandownload():
    dir ='downloads'
    try:
        for file in os.scandir(dir):
            os.remove(file.path)
    except:
        print("Failed Cleaning")

def play():
    while True:
        if musicplaylist.playlist:
            song=musicplaylist.playlistmanager()
            try:
                meta=musicplaylist.songdownloader(song)
            except:
                print(colored("Error downloading "+song,'red'))
            else:
                musicplaylist.songplayer(meta)
            cleandownload()
        else:
            pass

def queue():
    songrequest=""
    while songrequest!="exit":
        songrequest=input("")
        if songrequest=="spotify":
            spotifyplaylist=input("Enter Playlist: ")
            spotifyplaylist=downloader.spotifyparser(spotifyplaylist)
            musicplaylist.playlist.extend(spotifyplaylist)
        if songrequest=="youtube":
            ytplaylist=input("Enter Playlist: ")
            beg=1
            while beg!=-1:
                tempytplaylist,beg=downloader.ytplaylistparser(ytplaylist,beg)
                musicplaylist.playlist.extend(tempytplaylist)
                try:
                    playthread.start()
                except:
                    pass
        else:
            musicplaylist.songqueuer(songrequest)
        try:
            playthread.start()
        except:
            pass

playthread=threading.Thread(target=play,daemon=True)
queuethread=threading.Thread(target=queue)

def main():
    f = Figlet(font='banner3-D')
    print(" ")
    print(colored(f.renderText('JAKYM'),'cyan'))
    print("\t\t\t\t\t- by Lex")

    queuethread.start()
    queuethread.join()

    cleandownload()
    os.rmdir('downloads')


if __name__ == "__main__":
    main()



    



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jakym",
    version="0.3",
    author="Mayank Jha",
    author_email="jhamayank159@gmail.com",
    description="Just Another Konsole YouTube-Music",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/themayankjha/JAKYM",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'youtube_dl',
          'pyfiglet',
          'requests',
          'pydub',
          'termcolor',
          'beautifulsoup4',
          'lxml',
      ],
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['jakym = jakym.__main__:main']
    },
)
